@@ -1,5 +1,5 @@
 /*
-* Copyright (c) 2019-2021, Intel Corporation
+* Copyright (c) 2019-2022, Intel Corporation
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
@@ -29,6 +29,7 @@
 #include "vp_allocator.h"
 #include "vp_utils.h"
 #include "mos_solo_generic.h"
+#include "levelzero_npu_interface.h"
 
 using namespace vp;
 
@@ -164,8 +165,12 @@ MOS_STATUS VpAllocator::DestroySurface(MOS_SURFACE *surface, MOS_GFXRES_FREE_FLA
 {
     VP_FUNC_CALL();
     VP_PUBLIC_CHK_NULL_RETURN(m_allocator);
-
-    return m_allocator->DestroySurface(surface, flags);
+    MOS_GFXRES_FREE_FLAGS resFreeFlags = {0};
+    if (IsSyncFreeNeededForMMCSurface(surface))
+    {
+        resFreeFlags.SynchronousDestroy = 1;
+    }
+    return m_allocator->DestroySurface(surface, resFreeFlags);
 }
 
 VP_SURFACE* VpAllocator::AllocateVpSurface(MOS_ALLOC_GFXRES_PARAMS &param, bool zeroOnAllocate, VPHAL_CSPACE ColorSpace, uint32_t ChromaSiting)
@@ -227,9 +232,24 @@ VP_SURFACE *VpAllocator::AllocateVpSurface(VPHAL_SURFACE &vphalSurf)
     VP_FUNC_CALL();
     if (Mos_ResourceIsNull(&vphalSurf.OsResource))
     {
+        if (vphalSurf.pPipeIntermediateSurface)
+        {
+            VP_SURFACE *surf = MOS_New(VP_SURFACE);
+            if (surf == nullptr)
+            {
+                return nullptr;
+            }
+            surf->osSurface = MOS_New(MOS_SURFACE);
+            if (MOS_FAILED(CopyVpSurface(*surf, *vphalSurf.pPipeIntermediateSurface)))
+            {
+                MOS_Delete(surf->osSurface);
+                MOS_Delete(surf);
+                return nullptr;
+            }
+            return surf;
+        }
         return nullptr;
     }
-
     VP_SURFACE *surf = MOS_New(VP_SURFACE);
 
     if (nullptr == surf)
@@ -267,6 +287,44 @@ VP_SURFACE *VpAllocator::AllocateVpSurface(VPHAL_SURFACE &vphalSurf)
     // But it is mapped to several different Formats in CodecHal under different conditions.
     osSurface.Format                            = vphalSurf.Format;
 
+    // Add offset info
+    osSurface.dwOffset                    = vphalSurf.dwOffset;
+    osSurface.YPlaneOffset.iSurfaceOffset = vphalSurf.YPlaneOffset.iSurfaceOffset;
+    osSurface.YPlaneOffset.iXOffset       = vphalSurf.YPlaneOffset.iXOffset;
+    osSurface.YPlaneOffset.iYOffset       = vphalSurf.YPlaneOffset.iYOffset;
+    if (IS_RGB32_FORMAT(osSurface.Format) ||
+        IS_RGB16_FORMAT(osSurface.Format) ||
+        IS_RGB64_FORMAT(osSurface.Format) ||
+        osSurface.Format == Format_RGB ||
+        osSurface.Format == Format_Y410)
+    {
+        osSurface.dwOffset                    = vphalSurf.dwOffset;
+        osSurface.YPlaneOffset.iSurfaceOffset = vphalSurf.YPlaneOffset.iSurfaceOffset;
+        osSurface.YPlaneOffset.iXOffset       = vphalSurf.YPlaneOffset.iXOffset;
+        osSurface.YPlaneOffset.iYOffset       = vphalSurf.YPlaneOffset.iYOffset;
+    }
+    else  // YUV or PL3_RGB
+    {
+        // Get Y plane information (plane offset, X/Y offset)
+        osSurface.dwOffset                        = vphalSurf.dwOffset;
+        osSurface.YPlaneOffset.iSurfaceOffset     = vphalSurf.YPlaneOffset.iSurfaceOffset;
+        osSurface.YPlaneOffset.iXOffset           = vphalSurf.YPlaneOffset.iXOffset;
+        osSurface.YPlaneOffset.iYOffset           = vphalSurf.YPlaneOffset.iYOffset;
+        osSurface.YPlaneOffset.iLockSurfaceOffset = vphalSurf.YPlaneOffset.iLockSurfaceOffset;
+
+        // Get U/UV plane information (plane offset, X/Y offset)
+        osSurface.UPlaneOffset.iSurfaceOffset     = vphalSurf.UPlaneOffset.iSurfaceOffset;
+        osSurface.UPlaneOffset.iXOffset           = vphalSurf.UPlaneOffset.iXOffset;
+        osSurface.UPlaneOffset.iYOffset           = vphalSurf.UPlaneOffset.iYOffset;
+        osSurface.UPlaneOffset.iLockSurfaceOffset = vphalSurf.UPlaneOffset.iLockSurfaceOffset;
+
+        // Get V plane information (plane offset, X/Y offset)
+        osSurface.VPlaneOffset.iSurfaceOffset     = vphalSurf.VPlaneOffset.iSurfaceOffset;
+        osSurface.VPlaneOffset.iXOffset           = vphalSurf.VPlaneOffset.iXOffset;
+        osSurface.VPlaneOffset.iYOffset           = vphalSurf.VPlaneOffset.iYOffset;
+        osSurface.VPlaneOffset.iLockSurfaceOffset = vphalSurf.VPlaneOffset.iLockSurfaceOffset;
+    }
+
     // Initialize other parameters in vp surface according to vphal surface.
     surf->ColorSpace                            = vphalSurf.ColorSpace;
     surf->ExtendedGamut                         = vphalSurf.ExtendedGamut;
@@ -443,6 +501,29 @@ MOS_STATUS VpAllocator::DestroyVpSurface(VP_SURFACE* &surface, bool deferredDest
 
     if (surface->isResourceOwner)
     {
+#if !EMUL
+    MT_LOG5(MT_VP_HAL_DESTROY_SURF, MT_NORMAL,
+        MT_VP_HAL_SURF_ALLOC_PARAM_PTR, (int64_t)surface,
+        MT_VP_HAL_SURF_ALLOC_PARAM_MOS_SURF_PTR, (int64_t)surface->osSurface,
+        MT_VP_HAL_SURF_ALLOC_PARAM_IS_RES_OWNER, surface->isResourceOwner,
+        MT_VP_HAL_SURF_ALLOC_PARAM_HANDLE, static_cast<int64_t>(surface->GetAllocationHandle(m_osInterface)),
+        MT_VP_HAL_SURF_ALLOC_PARAM_SIZE, static_cast<int64_t>(surface->osSurface->OsResource.pGmmResInfo ? surface->osSurface->OsResource.pGmmResInfo->GetSizeAllocation() : 0));
+    VP_PUBLIC_NORMALMESSAGE(
+        "VP_HAL_DESTROY_SURF. "
+        "VP_HAL_Surface: %p, "
+        "VP_HAL_OsSurface: %p, "
+        "VP_HAL_isResourceOwner: %d, "
+        "VP_HAL_Surface_Handle: 0x%llx, "
+        "VP_HAL_Surface_Size: 0x%llx",
+        surface,
+        surface->osSurface,
+        surface->isResourceOwner ? 1 : 0,
+        surface->GetAllocationHandle(m_osInterface),
+        surface->osSurface->OsResource.pGmmResInfo ? surface->osSurface->OsResource.pGmmResInfo->GetSizeAllocation() : 0);
+
+        int64_t currentSize = static_cast<int64_t>(surface->osSurface->OsResource.pGmmResInfo ? surface->osSurface->OsResource.pGmmResInfo->GetSizeAllocation() : 0);
+        m_totalSize         = m_totalSize - currentSize;
+#endif 
         status = DestroySurface(surface->osSurface, flags);
     }
     else
@@ -550,6 +631,9 @@ MOS_STATUS VpAllocator::GetSurfaceInfo(VPHAL_SURFACE *surface, VPHAL_GET_SURFACE
     surface->dwWidth         = resDetails.dwWidth;
     surface->dwHeight        = resDetails.dwHeight;
     surface->dwPitch         = resDetails.dwPitch;
+    surface->dwYPitch        = resDetails.dwYPitch;
+    surface->dwUPitch        = resDetails.dwUPitch;
+    surface->dwVPitch        = resDetails.dwVPitch;
     surface->dwSlicePitch    = resDetails.dwSlicePitch;
     surface->dwDepth         = resDetails.dwDepth;
     surface->TileType        = resDetails.TileType;
@@ -560,6 +644,7 @@ MOS_STATUS VpAllocator::GetSurfaceInfo(VPHAL_SURFACE *surface, VPHAL_GET_SURFACE
     surface->Format          = resDetails.Format;
     surface->bCompressible   = resDetails.bCompressible ? true : false;
     surface->bIsCompressed   = resDetails.bIsCompressed ? true : false;
+    surface->CacheSetting    = resDetails.CacheSetting;
 
     if (IS_RGB32_FORMAT(surface->Format) ||
         IS_RGB16_FORMAT(surface->Format) ||
@@ -600,7 +685,8 @@ MOS_STATUS VpAllocator::GetSurfaceInfo(VPHAL_SURFACE *surface, VPHAL_GET_SURFACE
          surface->TileType == MOS_TILE_YS))
     {
         surface->bCompressible   = true;
-        surface->CompressionMode = (MOS_RESOURCE_MMC_MODE)mmcMode;
+        surface->CompressionMode = (mmcMode == MOS_MEMCOMP_MC) ? MOS_MMC_MC :
+                                   (mmcMode == MOS_MEMCOMP_RC) ? MOS_MMC_RC : MOS_MMC_DISABLED;
     }
     else
     {
@@ -656,6 +742,7 @@ MOS_STATUS VpAllocator::GetSurfaceInfo(VP_SURFACE* surface, VPHAL_GET_SURFACE_IN
     surface->osSurface->Format          = resDetails.Format;
     surface->osSurface->bCompressible   = resDetails.bCompressible ? true : false;
     surface->osSurface->bIsCompressed   = resDetails.bIsCompressed ? true : false;
+    surface->osSurface->CacheSetting    = resDetails.CacheSetting;
     
     return MOS_STATUS_SUCCESS;
 }
@@ -669,7 +756,7 @@ MOS_STATUS VpAllocator::AllocParamsInitType(
     VP_FUNC_CALL();
     VP_PUBLIC_CHK_NULL_RETURN(surface);
 
-#if !EMUL && !LINUX
+#if !EMUL && !LINUX && !ANDROID
     //  Need to reallocate surface according to expected tiletype instead of tiletype of the surface what we have
     if ( surface                           != nullptr &&
          surface->OsResource.pGmmResInfo   != nullptr &&
@@ -700,7 +787,7 @@ MOS_STATUS VpAllocator::AllocParamsInitType(
     //  Need to reallocate surface according to expected tiletype instead of tiletype of the surface what we have
     if (surface != nullptr                                      &&
         surface->osSurface != nullptr                           &&
-        surface->osSurface->OsResource.pGmmResInfo != nullptr   &&
+        !Mos_ResourceIsNull(&surface->osSurface->OsResource)    &&
         surface->osSurface->TileType == defaultTileType)
     {
         // Reallocate but use same tile type and resource type as current
@@ -717,6 +804,98 @@ MOS_STATUS VpAllocator::AllocParamsInitType(
     return MOS_STATUS_SUCCESS;
 }
 
+MOS_STATUS VpAllocator::DestroyNpuBuffer(
+    VP_SURFACE*& surface)
+{
+    VP_PUBLIC_CHK_NULL_RETURN(m_osInterface);
+    VP_PUBLIC_CHK_NULL_RETURN(m_osInterface->npuInterface);
+    VP_PUBLIC_CHK_NULL_RETURN(m_osInterface->npuInterface->pfnFreeHostMem);
+
+    MOS_GFXRES_FREE_FLAGS resFreeFlags = {0};
+    //if free the compressed surface, need set the sync dealloc flag as 1 for sync dealloc for aux table update
+    if (surface && IsSyncFreeNeededForMMCSurface(surface->osSurface))
+    {
+        resFreeFlags.SynchronousDestroy = 1;
+        VP_PUBLIC_NORMALMESSAGE("Set SynchronousDestroy flag for compressed resource");
+    }
+    void *npuHostMem = surface ? surface->zeNpuHostMem : nullptr;
+    MOS_STATUS eStatus    = MOS_STATUS_SUCCESS;
+    eStatus = DestroyVpSurface(surface, false, resFreeFlags);
+    VP_PUBLIC_CHK_STATUS_RETURN(m_osInterface->npuInterface->pfnFreeHostMem(m_osInterface->npuInterface, npuHostMem));
+
+    return eStatus;
+}
+
+MOS_STATUS VpAllocator::ReAllocateNpuBuffer(
+    VP_SURFACE *&surface,
+    PCCHAR       surfaceName,
+    uint32_t     size,
+    bool        &allocated)
+{
+    VP_FUNC_CALL();
+
+    allocated = false;
+    VP_PUBLIC_CHK_NULL_RETURN(m_osInterface);
+    VP_PUBLIC_CHK_NULL_RETURN(m_osInterface->npuInterface);
+    VP_PUBLIC_CHK_NULL_RETURN(m_osInterface->npuInterface->pfnAllocateHostMem);
+    VP_PUBLIC_CHK_NULL_RETURN(m_osInterface->npuInterface->pfnFreeHostMem);
+
+    auto surfInfoCheck = [=](VP_SURFACE *&surface) -> bool {
+        return (surface->bufferWidth == size) && (Format_Buffer == surface->osSurface->Format);
+    };
+
+    if (surface &&
+        surface->osSurface &&
+        !Mos_ResourceIsNull(&surface->osSurface->OsResource) &&
+        surface->zeNpuHostMem != nullptr &&
+        surfInfoCheck(surface))
+    {
+        return MOS_STATUS_SUCCESS;
+    }
+
+    if (surface && nullptr == surface->osSurface)
+    {
+        // VP_SURFACE should always be allocated by interface in VpAllocator,
+        // which will ensure nullptr != surface->osSurface.
+        VP_PUBLIC_CHK_STATUS_RETURN(MOS_STATUS_INVALID_PARAMETER);
+    }
+
+    VP_PUBLIC_CHK_STATUS_RETURN(DestroyNpuBuffer(surface));
+    
+    surface = AllocateVpSurface();
+    VP_PUBLIC_CHK_NULL_RETURN(surface);
+    if (surface->osSurface == nullptr)
+    {
+        VP_PUBLIC_CHK_STATUS_RETURN(DestroyVpSurface(surface));
+        VP_PUBLIC_CHK_STATUS_RETURN(MOS_STATUS_NO_SPACE);
+    }
+
+    void *npuHostMem = nullptr;
+    VP_PUBLIC_CHK_STATUS_RETURN(m_osInterface->npuInterface->pfnAllocateHostMem(m_osInterface->npuInterface, size, npuHostMem));
+    VP_PUBLIC_CHK_NULL_RETURN(npuHostMem);
+
+    GMM_RESOURCE_FORMAT gmmFormat = {};
+#if !EMUL
+    gmmFormat = GMM_FORMAT_GENERIC_8BIT;
+#endif
+    if (MOS_STATUS_SUCCESS != AllocateCPUResource(&surface->osSurface->OsResource, (size_t)npuHostMem, size, 1, size, size, 0, gmmFormat))
+    {
+        VP_PUBLIC_CHK_STATUS_RETURN(m_osInterface->npuInterface->pfnFreeHostMem(m_osInterface->npuInterface, npuHostMem));
+        VP_PUBLIC_CHK_STATUS_RETURN(DestroyVpSurface(surface));
+        VP_PUBLIC_CHK_STATUS_RETURN(MOS_STATUS_NO_SPACE);
+    }
+    surface->zeNpuHostMem    = npuHostMem;
+    surface->isResourceOwner = true;
+    surface->osSurface->dwWidth  = size;
+    surface->osSurface->dwHeight = 1;
+    surface->osSurface->dwSize   = size;
+    surface->osSurface->TileType = MOS_TILE_LINEAR;
+    surface->osSurface->Format   = Format_Buffer;
+
+    allocated = true;
+    return MOS_STATUS_SUCCESS;
+}
+
 MOS_STATUS VpAllocator::ReAllocateSurface(
         VP_SURFACE             *&surface,
         PCCHAR                  surfaceName,
@@ -733,7 +912,9 @@ MOS_STATUS VpAllocator::ReAllocateSurface(
         MOS_HW_RESOURCE_DEF     resUsageType,
         MOS_TILE_MODE_GMM       tileModeByForce,
         Mos_MemPool             memType,
-        bool                    isNotLockable)
+        bool                    isNotLockable,
+        void *                  systemMemory,
+        uint32_t                depth)
 {
     VP_FUNC_CALL();
     MOS_STATUS              eStatus = MOS_STATUS_SUCCESS;
@@ -745,28 +926,47 @@ MOS_STATUS VpAllocator::ReAllocateSurface(
     //---------------------------------
     VP_PUBLIC_CHK_NULL_RETURN(m_allocator);
 
-    if (!m_mmc->IsMmcEnabled())
+    if (!m_mmc->IsMmcEnabled() || 
+        !m_mmc->IsCompressibelSurfaceSupported())
     {
         compressible    = 0;
         compressionMode = MOS_MMC_DISABLED;
     }
     //---------------------------------
 
+    auto surfInfoCheck = [=](VP_SURFACE *&surface) -> bool
+                            {
+                                return  (surface->osSurface->Format               == format)          &&
+                                        ((surface->osSurface->bCompressible != 0) == compressible)    &&
+                                        (surface->osSurface->CompressionMode      == compressionMode) &&
+                                        (surface->osSurface->TileType             == defaultTileType  ||
+                                        MOS_TILE_Y                                == defaultTileType  &&
+                                        IS_Y_MAJOR_TILE_FORMAT(surface->osSurface->TileType))         &&
+                                        ((Format_Buffer                           == format           &&
+                                        surface->bufferWidth                      == width            &&
+                                        surface->bufferHeight                     == height)          ||
+                                        (Format_Buffer                            != format           &&
+                                        surface->osSurface->dwWidth               == width            &&
+                                        surface->osSurface->dwHeight              == height)           );
+                            };
+
     // compressible should be compared with bCompressible since it is inited by bCompressible in previous call
     // TileType of surface should be compared since we need to reallocate surface if TileType changes
     if (surface                                                       &&
         surface->osSurface                                            &&
         !Mos_ResourceIsNull(&surface->osSurface->OsResource)          &&
-        (surface->osSurface->Format               == format)          &&
-        ((surface->osSurface->bCompressible != 0) == compressible)    &&
-        (surface->osSurface->CompressionMode      == compressionMode) &&
-        (surface->osSurface->TileType             == defaultTileType) &&
-        ((Format_Buffer                           == format           &&
-          surface->bufferWidth                    == width            &&
-          surface->bufferHeight                   == height)          ||
-         (Format_Buffer                           != format           &&
-          surface->osSurface->dwWidth             == width            &&
-          surface->osSurface->dwHeight            == height))         )
+        surfInfoCheck(surface)                                        )
+    {
+        return eStatus;
+    }
+
+    // reuse the allocated buffer if the allocated size was larger than request size when OptimizeCpuTiming is enabled
+    if (m_osInterface->bOptimizeCpuTiming                               &&
+        surface                                                         &&
+        surface->osSurface                                              &&
+        (!Mos_ResourceIsNull(&surface->osSurface->OsResource))          &&
+        (Format_Buffer                                == format)        &&
+        (surface->bufferWidth * surface->bufferHeight >= width * height))
     {
         return eStatus;
     }
@@ -790,6 +990,7 @@ MOS_STATUS VpAllocator::ReAllocateSurface(
 
     allocParams.dwWidth         = width;
     allocParams.dwHeight        = height;
+    allocParams.dwDepth         = depth;
     allocParams.Format          = format;
     allocParams.bIsCompressible = compressible;
     allocParams.CompressionMode = compressionMode;
@@ -799,14 +1000,67 @@ MOS_STATUS VpAllocator::ReAllocateSurface(
     allocParams.m_tileModeByForce = tileModeByForce;
     allocParams.dwMemType       = memType;
     allocParams.Flags.bNotLockable = isNotLockable;
+    allocParams.pSystemMemory      = systemMemory;
 
     surface = AllocateVpSurface(allocParams, zeroOnAllocate);
     VP_PUBLIC_CHK_NULL_RETURN(surface);
+    VP_PUBLIC_CHK_NULL_RETURN(surface->osSurface);
+    if (Mos_ResourceIsNull(&surface->osSurface->OsResource))
+    {
+        VP_PUBLIC_CHK_NULL_RETURN(nullptr);
+    }
+
+    if (!surfInfoCheck(surface))
+    {
+        VP_PUBLIC_ASSERTMESSAGE("Allocated surface is not matched with allocating parameter.");
+        VP_PUBLIC_ASSERTMESSAGE("Allocated surface : format %d, compressible %d, compressionMode %d, tileType %d, bufferWidth %d, bufferHeight %d, width %d, height %d",
+            surface->osSurface->Format,
+            surface->osSurface->bCompressible,
+            surface->osSurface->CompressionMode,
+            surface->osSurface->TileType,
+            surface->bufferWidth,
+            surface->bufferHeight,
+            surface->osSurface->dwWidth,
+            surface->osSurface->dwHeight);
+        VP_PUBLIC_ASSERTMESSAGE("Parameter to allocate : format %d, compressible %d, compressionMode %d, tileType %d, width %d, height %d",
+            format,
+            compressible,
+            compressionMode,
+            defaultTileType,
+            width,
+            height);
+    }
 
     MT_LOG7(MT_VP_HAL_REALLOC_SURF, MT_NORMAL, MT_VP_HAL_INTER_SURF_TYPE, surfaceName ? *((int64_t*)surfaceName) : 0,
         MT_SURF_WIDTH, width, MT_SURF_HEIGHT, height, MT_SURF_MOS_FORMAT, format, MT_SURF_TILE_MODE, surface->osSurface->TileModeGMM,
         MT_SURF_COMP_ABLE, surface->osSurface->bCompressible, MT_SURF_COMP_MODE, surface->osSurface->CompressionMode);
 
+#if !EMUL
+    MT_LOG6(MT_VP_HAL_REALLOC_SURF, MT_NORMAL,
+        MT_VP_HAL_SURF_ALLOC_PARAM_PTR, (int64_t)surface,
+        MT_VP_HAL_SURF_ALLOC_PARAM_MOS_SURF_PTR, (int64_t)surface->osSurface,
+        MT_VP_HAL_SURF_ALLOC_PARAM_IS_RES_OWNER, surface->isResourceOwner,
+        MT_VP_HAL_SURF_ALLOC_PARAM_HANDLE, static_cast<int64_t>(surface->GetAllocationHandle(m_osInterface)),
+        MT_VP_HAL_SURF_ALLOC_PARAM_SIZE, static_cast<int64_t>(surface->osSurface->OsResource.pGmmResInfo ? surface->osSurface->OsResource.pGmmResInfo->GetSizeAllocation() : 0),
+        MT_VP_HAL_SURF_ALLOC_PARAM_NAME, surfaceName ? *((int64_t *)surfaceName) : 0);
+    VP_PUBLIC_NORMALMESSAGE(
+        "VP_HAL_REALLOC_SURF. "
+        "VP_HAL_Surface: %p, "
+        "VP_HAL_OsSurface: %p, "
+        "VP_HAL_isResourceOwner: %d, "
+        "VP_HAL_Surface_Handle: 0x%llx, "
+        "VP_HAL_Surface_Size: 0x%llx, "
+        "VP_HAL_Surface_Name: %s",
+        surface,
+        surface->osSurface,
+        surface->isResourceOwner ? 1 : 0,
+        surface->GetAllocationHandle(m_osInterface),
+        surface->osSurface->OsResource.pGmmResInfo ? surface->osSurface->OsResource.pGmmResInfo->GetSizeAllocation() : 0,
+        surfaceName ? surfaceName : "");
+    int64_t currentSize  = static_cast<int64_t>(surface->osSurface->OsResource.pGmmResInfo ? surface->osSurface->OsResource.pGmmResInfo->GetSizeAllocation() : 0);
+    m_totalSize          = m_totalSize + currentSize;
+    m_peakSize           = m_peakSize > m_totalSize ? m_peakSize : m_totalSize;
+#endif
     allocated = true;
     return MOS_STATUS_SUCCESS;
 }
@@ -853,6 +1107,15 @@ MOS_STATUS VpAllocator::ReAllocateSurface(
         return eStatus;
     }
 
+    // reuse the allocated buffer if the allocated size was larger than request size when OptimizeCpuTiming is enabled
+    if (m_osInterface->bOptimizeCpuTiming                       &&
+        (!Mos_ResourceIsNull(&surface->OsResource))           &&
+        (Format_Buffer                        == format)        &&
+        (surface->dwWidth * surface->dwHeight >= width * height))
+    {
+        return eStatus;
+    }
+
     MOS_ZeroMemory(&allocParams, sizeof(MOS_ALLOC_GFXRES_PARAMS));
 
     VpHal_AllocParamsInitType(&allocParams, surface, defaultResType, defaultTileType);
@@ -887,6 +1150,67 @@ MOS_STATUS VpAllocator::ReAllocateSurface(
 }
 #endif
 
+MOS_STATUS VpAllocator::ReAllocateVpSurfaceWithSameConfigOfVphalSurface(
+    VP_SURFACE          *&surface,
+    const PVPHAL_SURFACE &vphalSurface,
+    PCCHAR                surfaceName,
+    bool                 &allocated)
+{
+    VP_PUBLIC_CHK_NULL_RETURN(vphalSurface);
+
+    MOS_GFXRES_TYPE     defaultResType  = MOS_GFXRES_INVALID;
+    MOS_TILE_TYPE       defaultTileType = MOS_TILE_LINEAR;
+    MOS_HW_RESOURCE_DEF resUsageType    = MOS_HW_RESOURCE_USAGE_VP_INTERNAL_READ_WRITE_RENDER;
+    MOS_TILE_MODE_GMM   tileModeByForce = MOS_TILE_UNSET_GMM;
+    if (Mos_ResourceIsNull(&vphalSurface->OsResource))
+    {
+        bool isBuffer   = (vphalSurface->Format == Format_Buffer || vphalSurface->Format == Format_RAW);
+        defaultResType  = isBuffer ? MOS_GFXRES_BUFFER : MOS_GFXRES_2D;
+        defaultTileType = isBuffer ? MOS_TILE_LINEAR : MOS_TILE_Y;
+    }
+    else
+    {
+        defaultResType = m_osInterface->pfnGetResType(&vphalSurface->OsResource);
+        defaultTileType = vphalSurface->OsResource.TileType;
+        resUsageType    = MOS_HW_RESOURCE_USAGE_VP_INTERNAL_READ_WRITE_RENDER;
+        tileModeByForce = MOS_TILE_UNSET_GMM;
+    }
+    VP_PUBLIC_CHK_STATUS_RETURN(ReAllocateSurface(
+        surface,
+        surfaceName,
+        vphalSurface->Format,
+        defaultResType,
+        defaultTileType,
+        vphalSurface->dwWidth,
+        vphalSurface->dwHeight,
+        vphalSurface->bCompressible,
+        vphalSurface->CompressionMode,
+        allocated,
+        false,
+        false,
+        resUsageType,
+        tileModeByForce));
+
+    // Initialize other parameters in vp surface according to vphal surface.
+    surface->ColorSpace     = vphalSurface->ColorSpace;
+    surface->ExtendedGamut  = vphalSurface->ExtendedGamut;
+    surface->Palette        = vphalSurface->Palette;
+    surface->bQueryVariance = vphalSurface->bQueryVariance;
+    surface->FrameID        = vphalSurface->FrameID;
+    surface->uFwdRefCount   = vphalSurface->uFwdRefCount;
+    surface->uBwdRefCount   = vphalSurface->uBwdRefCount;
+    surface->pFwdRef        = vphalSurface->pFwdRef;
+    surface->pBwdRef        = vphalSurface->pBwdRef;
+    surface->SurfType       = vphalSurface->SurfType;
+    surface->SampleType     = vphalSurface->SampleType;
+    surface->ChromaSiting   = vphalSurface->ChromaSiting;
+    surface->rcSrc          = vphalSurface->rcSrc;
+    surface->rcDst          = vphalSurface->rcDst;
+    surface->rcMaxSrc       = vphalSurface->rcMaxSrc;
+
+    return MOS_STATUS_SUCCESS;
+}
+
 MOS_STATUS VpAllocator::OsFillResource(
     PMOS_RESOURCE     osResource,
     uint32_t          size,
@@ -1156,6 +1480,68 @@ void VpAllocator::CleanRecycler()
     }
 }
 
+MOS_STATUS VpAllocator::AllocateCPUResource(PMOS_RESOURCE osResource, size_t linearAddress, uint32_t dataSize, uint32_t height, uint64_t width, uint64_t planePitch, uint32_t CpTag, GMM_RESOURCE_FORMAT Format)
+{
+    VP_FUNC_CALL();
+    MOS_STATUS eStatusKey = MOS_STATUS_SUCCESS;
+    //Create MOS_GFXRES_BUFFER type resource
+    
+    size_t   linearAddressAligned = 0;
+    uint32_t addedShiftLeftOffset = 0;
+    MOS_ALLOC_GFXRES_PARAMS sParams;
+
+    if ((linearAddress & 0xf) || (linearAddress == 0))
+    {
+        VP_PUBLIC_NORMALMESSAGE("Error: Start address of system memory is not 16-byte aligned!");
+        return MOS_STATUS_UNKNOWN;
+    }
+    if (sizeof(void *) == 8)  //64-bit
+    {
+        linearAddressAligned = linearAddress & ADDRESS_PAGE_ALIGNMENT_MASK_X64;
+    }
+    else  //32-bit
+    {
+        linearAddressAligned = linearAddress & ADDRESS_PAGE_ALIGNMENT_MASK_X86;
+    }
+    //Calculate  Left Shift offset
+    addedShiftLeftOffset = (uint32_t)(linearAddress - linearAddressAligned);
+    VP_PUBLIC_NORMALMESSAGE("System memory address: 0x%x, Aligned address: 0x%x, offset: %u", linearAddress, linearAddressAligned, addedShiftLeftOffset);
+
+    MOS_ZeroMemory(&sParams, sizeof(MOS_ALLOC_GFXRES_PARAMS));
+    sParams.Type           = MOS_GFXRES_BUFFER;
+    sParams.dwBytes        = dataSize + addedShiftLeftOffset;
+    sParams.pSystemMemory  = (void *)linearAddressAligned;
+    sParams.TileType       = MOS_TILE_LINEAR;
+    sParams.Format         = Format_Buffer;
+    sParams.pBufName       = "cpuResourceBuffer_1D";
+    sParams.bBypassMODImpl = 1;
+    sParams.bIsPersistent  = 1;
+    
+    eStatusKey = m_osInterface->pfnAllocateResource(m_osInterface, &sParams, osResource);
+    if (eStatusKey != MOS_STATUS_SUCCESS)
+    {
+        VP_PUBLIC_NORMALMESSAGE("Error: m_osInterface.pfnAllocateResource failed!");
+        return eStatusKey;
+    }
+#if !EMUL
+    osResource->pGmmResInfo->GetSetCpSurfTag(true, CpTag);
+    osResource->pGmmResInfo->OverridePitch(planePitch);
+
+    osResource->pGmmResInfo->OverrideSurfaceFormat(Format);
+    osResource->pGmmResInfo->OverrideSurfaceType(RESOURCE_2D);
+    osResource->pGmmResInfo->OverrideBaseWidth(width);
+    osResource->pGmmResInfo->OverrideBaseHeight(height);
+
+    osResource->pGmmResInfo->OverridePlanarXOffset(GMM_NO_PLANE, (GMM_GFX_SIZE_T)addedShiftLeftOffset);
+    osResource->pGmmResInfo->OverridePlanarYOffset(GMM_NO_PLANE, 0);
+    osResource->pGmmResInfo->OverridePlanarXOffset(GMM_PLANE_Y, (GMM_GFX_SIZE_T)addedShiftLeftOffset);
+    osResource->pGmmResInfo->OverridePlanarYOffset(GMM_PLANE_Y, 0);
+#endif
+
+    return eStatusKey;
+
+}
+
 bool VP_SURFACE::IsEmpty()
 {
     VP_FUNC_CALL();
@@ -1199,9 +1585,12 @@ MOS_STATUS VP_SURFACE::Clean()
 uint64_t VP_SURFACE::GetAllocationHandle(MOS_INTERFACE* osIntf)
 {
     VP_FUNC_CALL();
-
+    if (nullptr == osIntf)
+    {
+        return 0;
+    }
 #if MOS_MEDIASOLO_SUPPORTED
-    if (Mos_Solo_IsInUse(osIntf))
+    if (osIntf && osIntf->bSoloInUse)
     {
         uint64_t handle = osSurface ? (uint64_t)osSurface->OsResource.pData : 0;
         if (handle)
@@ -1213,18 +1602,11 @@ uint64_t VP_SURFACE::GetAllocationHandle(MOS_INTERFACE* osIntf)
     }
 #endif
 
-#if(LINUX)
-    if (osSurface && osSurface->OsResource.bo)
-    {
-        return osSurface->OsResource.bo->handle;
-    }
-    else
+    if (nullptr == osSurface || nullptr == osIntf)
     {
         return 0;
     }
-#else
-    return osSurface ? osSurface->OsResource.AllocationInfo.m_AllocationHandle : 0;
-#endif
+    return osIntf->pfnGetResourceHandle(osIntf->osStreamState, &osSurface->OsResource);
 }
 
 MOS_STATUS VpAllocator::SetMmcFlags(MOS_SURFACE &osSurface)
@@ -1242,7 +1624,7 @@ MOS_STATUS VpAllocator::SetMmcFlags(MOS_SURFACE &osSurface)
 
         osSurface.bCompressible   = true;
         osSurface.bIsCompressed   = true;
-        m_mmc->GetSurfaceMmcFormat(&osSurface, &mmcFormat);
+        VP_PUBLIC_CHK_STATUS_RETURN(m_mmc->GetSurfaceMmcFormat(&osSurface, &mmcFormat));
         osSurface.CompressionFormat = mmcFormat;
     }
     else
@@ -1256,3 +1638,18 @@ MOS_STATUS VpAllocator::SetMmcFlags(MOS_SURFACE &osSurface)
 
     return MOS_STATUS_SUCCESS;
 }
+
+MOS_HW_RESOURCE_DEF VpAllocator::GetResourceCache(uint32_t feature, bool bOut, ENGINE_TYPE engineType, MOS_COMPONENT id)
+{
+    VP_FUNC_CALL();
+    MOS_CACHE_ELEMENT element(MOS_CODEC_RESOURCE_USAGE_BEGIN_CODEC, MOS_CODEC_RESOURCE_USAGE_BEGIN_CODEC);
+    bool              res = m_osInterface->pfnGetCacheSetting(id, SUFACE_TYPE_ASSIGNED(feature), bOut, engineType, element, false);
+    if (res == false)
+    {
+        VP_PUBLIC_ASSERTMESSAGE("Resource %u was not found in cache manager, use default usage MOS_HW_RESOURCE_USAGE_VP_INTERNAL_READ_WRITE_RENDER!", feature);
+        return MOS_HW_RESOURCE_USAGE_VP_INTERNAL_READ_WRITE_RENDER;
+    }
+
+    VP_PUBLIC_NORMALMESSAGE("Resource %u was found in cache manager, use mocs usage %u!", feature, element.mocsUsageType);
+    return element.mocsUsageType;
+}
